/*-------------------------------------------------- 2014-02-06 ----*/
/* arduino stepper motor shield STACk version V1.00                 */
/* Driver TB6560AHQ Max3.5A                                         */
/*------------------------------------------------------------------*/
#include <MsTimer2.h>

#define MOTOR_X_CLK     13  // D13 
#define MOTOR_Y_CLK     12  // D12
#define MOTOR_Z_CLK     11  // D11
#define MOTOR_X_RESET   10  // D10
#define MOTOR_Y_RESET    9  // D9
#define MOTOR_Z_RESET    8  // D8
#define MOTOR_X_ENABLE   7  // D7
#define MOTOR_Y_ENABLE   6  // D6
#define MOTOR_Z_ENABLE   5  // D5
#define MOTOR_X_CWCCW   14  // A0 
#define MOTOR_Y_CWCCW   15  // A1 
#define MOTOR_Z_CWCCW   16  // A2
#define MOTOR_X_TQ1     17  // A3
#define MOTOR_Y_TQ1     18  // A4
#define MOTOR_Z_TQ1     19  // A5
#define MOTOR_X_TQ2      4  // D4
#define MOTOR_Y_TQ2      3  // D3
#define MOTOR_Z_TQ2      2  // D2

#define MOTOR_X    'X'
#define MOTOR_Y    'Y'
#define MOTOR_Z    'Z'
#define MOTOR_ALL  'A'

#define TORQUE_100 '4'   // torque 100%
#define TORQUE_75  '3'   // torque  75%
#define TORQUE_50  '2'   // torque  50%
#define TORQUE_25  '1'   // torque  25%

#define CCW  HIGH
#define CW   LOW

//------ gloval variable --------------------
volatile boolean interpret_progress;
volatile unsigned long step_x;
volatile unsigned long step_y;
volatile unsigned long step_z;
volatile word speed_x;
volatile word speed_y;
volatile word speed_z;
volatile word speed_x_counter;
volatile word speed_y_counter;
volatile word speed_z_counter;
volatile byte rotation_direction_x;
volatile byte rotation_direction_y;
volatile byte rotation_direction_z;
volatile byte stat_x;
volatile byte stat_y;
volatile byte stat_z;
#define STATUS_RUN   0
#define STATUS_STOP  1

//-----------------------------------------------------------------
// stop motor
//-----------------------------------------------------------------
void TB6560_power_off(byte motor)
{
  switch( motor ) {
    case MOTOR_X:
      digitalWrite( MOTOR_X_RESET,  LOW );
      digitalWrite( MOTOR_X_ENABLE, LOW );
//      Serial.println("Motor X OFF");
      break;
    case MOTOR_Y:
      digitalWrite( MOTOR_Y_RESET,  LOW );
      digitalWrite( MOTOR_Y_ENABLE, LOW );
//      Serial.println("Motor Y OFF");
      break;
    case MOTOR_Z:
      digitalWrite( MOTOR_Z_RESET,  LOW );
      digitalWrite( MOTOR_Z_ENABLE, LOW );
//      Serial.println("Motor Z OFF");
      break;
    case MOTOR_ALL:
      digitalWrite( MOTOR_X_RESET,  LOW );
      digitalWrite( MOTOR_Y_RESET,  LOW );
      digitalWrite( MOTOR_Z_RESET,  LOW );     
      digitalWrite( MOTOR_X_ENABLE, LOW );
      digitalWrite( MOTOR_Y_ENABLE, LOW );
      digitalWrite( MOTOR_Z_ENABLE, LOW );
//      Serial.println("Motor ALL OFF");
      break;
  }
}

//-----------------------------------------------------------------
// start motor
//-----------------------------------------------------------------
void TB6560_power_on(byte motor)
{
  switch( motor ) {
    case MOTOR_X:
      digitalWrite( MOTOR_X_RESET,  HIGH );
      digitalWrite( MOTOR_X_ENABLE, HIGH );
//      Serial.println("Motor X ON");
      break;
    case MOTOR_Y:
      digitalWrite( MOTOR_Y_RESET,  HIGH );
      digitalWrite( MOTOR_Y_ENABLE, HIGH );
//      Serial.println("Motor Y ON");
      break;
    case MOTOR_Z:
      digitalWrite( MOTOR_Z_RESET,  HIGH );
      digitalWrite( MOTOR_Z_ENABLE, HIGH );
//      Serial.println("Motor Z ON");
      break;
    case MOTOR_ALL:
      digitalWrite( MOTOR_X_RESET,  HIGH );
      digitalWrite( MOTOR_Y_RESET,  HIGH );
      digitalWrite( MOTOR_Z_RESET,  HIGH );     
      digitalWrite( MOTOR_X_ENABLE, HIGH );
      digitalWrite( MOTOR_Y_ENABLE, HIGH );
      digitalWrite( MOTOR_Z_ENABLE, HIGH );
//      Serial.println("Motor ALL ON");
      break;
  }
}

//-----------------------------------------------------------------
// set torque
//-----------------------------------------------------------------
void TB6560_torgue(byte motor, byte torque)
{
  switch( motor ) {
    case MOTOR_X: TB6560_set_torgue(torque, MOTOR_X_TQ1,MOTOR_X_TQ2); break;
    case MOTOR_Y: TB6560_set_torgue(torque, MOTOR_Y_TQ1,MOTOR_Y_TQ2); break;
    case MOTOR_Z: TB6560_set_torgue(torque, MOTOR_Z_TQ1,MOTOR_Z_TQ2); break;
    case MOTOR_ALL:
      TB6560_set_torgue(torque, MOTOR_X_TQ1,MOTOR_X_TQ2);
      TB6560_set_torgue(torque, MOTOR_Y_TQ1,MOTOR_Y_TQ2);
      TB6560_set_torgue(torque, MOTOR_Z_TQ1,MOTOR_Z_TQ2);
      break;
  }
}

void TB6560_set_torgue(byte torque, byte tq1_port, byte tq2_port)
{
  switch( torque ) {
    case TORQUE_100: digitalWrite(tq1_port, LOW ); digitalWrite(tq2_port, LOW ); break;
    case TORQUE_75:  digitalWrite(tq1_port, HIGH); digitalWrite(tq2_port, LOW ); break;
    case TORQUE_50:  digitalWrite(tq1_port, LOW ); digitalWrite(tq2_port, HIGH); break;
    case TORQUE_25:  digitalWrite(tq1_port, HIGH); digitalWrite(tq2_port, HIGH); break;
  }
}

//-----------------------------------------------------------------
// set parameter
//-----------------------------------------------------------------
void TB6560_parameter( byte m_number, byte m_direction, unsigned long m_step, word m_speed )
{
  switch( m_number ) {
    case MOTOR_X:
      rotation_direction_x = m_direction;
      step_x = m_step;
      speed_x = m_speed;
      speed_x_counter = m_speed;
      break;
    case MOTOR_Y:
      rotation_direction_y = m_direction;
      step_y = m_step;
      speed_y = m_speed;
      speed_y_counter = m_speed;
      break;
    case MOTOR_Z:
      rotation_direction_z = m_direction;
      step_z = m_step;
      speed_z = m_speed;
      speed_z_counter = m_speed;
      break;
    case MOTOR_ALL:
      rotation_direction_x = m_direction;
      rotation_direction_y = m_direction;
      rotation_direction_z = m_direction;
      step_x = m_step;
      step_y = m_step;
      step_z = m_step;
      speed_x = m_speed;
      speed_y = m_speed;
      speed_z = m_speed;
      speed_x_counter = m_speed;
      speed_y_counter = m_speed;
      speed_z_counter = m_speed;
      break;
  }
}

//-----------------------------------------------------------------
// execute command
//-----------------------------------------------------------------
void exec_command()
{
  byte stepper_cmd;
  byte stepper_motor_number;
  byte stepper_motor_dirtq;   // direction or torque
  unsigned int  stepper_motor_speed;
  unsigned long stepper_motor_step;

  char serial_buf_speed[11];
  char serial_buf_step[6];
  char rbyte;
  byte index;
  byte string_index;
  boolean is_speed = false;
  
  interpret_progress = true;
  index = 0;
  string_index = 0;
  while(true) {
    if (Serial.available() > 0) {
      rbyte = Serial.read();
      if(rbyte == '#' ) {
        serial_buf_speed[string_index] = '\0';
        Serial.write('A');
        break;
      }
      switch( index ) {
        case 0: stepper_cmd          = rbyte; break;
        case 1: stepper_motor_number = rbyte; break;
        case 2: stepper_motor_dirtq  = rbyte; break;
        default:
          if( rbyte == ',' ) {
            is_speed = true;
            serial_buf_step[string_index] = '\0';
            string_index = 0;
            break;
          }
          if( is_speed ) {
            serial_buf_speed[string_index] = rbyte;
            string_index++;
          } else {
            serial_buf_step[string_index] = rbyte;
            string_index++;
          }
          break;
      }       
      index++;
    }
  }

  switch( stepper_motor_number ) {
    case MOTOR_X:
      interpret_progress = false;
      while( stat_x == STATUS_RUN ) {}; 
      interpret_progress = true;
      break;
    case MOTOR_Y:
      interpret_progress = false;
      while( stat_y == STATUS_RUN ) {};
      interpret_progress = true;
      break;
    case MOTOR_Z:
      interpret_progress = false;
      while( stat_z == STATUS_RUN ) {}; 
      interpret_progress = true;
      break;
    case MOTOR_ALL:
      interpret_progress = false;
      while( stat_x == STATUS_RUN ) {}; 
      while( stat_y == STATUS_RUN ) {}; 
      while( stat_z == STATUS_RUN ) {}; 
      interpret_progress = true;
      break;
  }

  switch( stepper_cmd ) {
    case 'P':      // set Parameter
      stepper_motor_step  = atol(serial_buf_step);
      stepper_motor_speed = atoi(serial_buf_speed);
      TB6560_parameter( stepper_motor_number, stepper_motor_dirtq, stepper_motor_step, stepper_motor_speed );
      break;
    case 'T':     // set Torque
      TB6560_torgue(stepper_motor_number, stepper_motor_dirtq);
      break;
    case 'R':     // Run 
      switch( stepper_motor_number ) {
        case MOTOR_X: if(step_x != 0 ) {stat_x = STATUS_RUN;} break;
        case MOTOR_Y: if(step_y != 0 ) {stat_y = STATUS_RUN;} break;
        case MOTOR_Z: if(step_z != 0 ) {stat_z = STATUS_RUN;} break;
        case MOTOR_ALL:
          if(step_x != 0 ) {stat_x = STATUS_RUN;}
          if(step_y != 0 ) {stat_y = STATUS_RUN;}
          if(step_z != 0 ) {stat_z = STATUS_RUN;}
          break;
      }
      break;
    case 'S':    // Stop
      switch( stepper_motor_number ) {
        case MOTOR_X: stat_x = STATUS_STOP; break;
        case MOTOR_Y: stat_y = STATUS_STOP; break;
        case MOTOR_Z: stat_z = STATUS_STOP; break;
        case MOTOR_ALL:
          stat_x = STATUS_STOP;
          stat_y = STATUS_STOP;
          stat_z = STATUS_STOP;
          break;
      }
      break;
    case 'O':   // motor power On
      switch( stepper_motor_number ) {
        case MOTOR_X: TB6560_power_on( MOTOR_X ); break;
        case MOTOR_Y: TB6560_power_on( MOTOR_Y ); break;
        case MOTOR_Z: TB6560_power_on( MOTOR_Z ); break;
      }
      break;
    case 'F':   // motor power oFf
      switch( stepper_motor_number ) {
        case MOTOR_X: TB6560_power_off( MOTOR_X ); break;
        case MOTOR_Y: TB6560_power_off( MOTOR_Y ); break;
        case MOTOR_Z: TB6560_power_off( MOTOR_Z ); break;
      }
      break;
    default:
//      Serial.println("COMMAND ERROR");
     break;
  }
  
  interpret_progress = false;
}

//-----------------------------------------------------------------
// timer interrupt
//-----------------------------------------------------------------
void move_step()
{
  if( interpret_progress ) {
    exit;
  }

  if( rotation_direction_x == 'M' ) { digitalWrite( MOTOR_X_CWCCW, CW ); } else { digitalWrite( MOTOR_X_CWCCW, CCW ); }  
  if( rotation_direction_y == 'M' ) { digitalWrite( MOTOR_Y_CWCCW, CW ); } else { digitalWrite( MOTOR_Y_CWCCW, CCW ); }
  if( rotation_direction_z == 'M' ) { digitalWrite( MOTOR_Z_CWCCW, CW ); } else { digitalWrite( MOTOR_Z_CWCCW, CCW ); }
  
  if( stat_x == STATUS_RUN ) {
    if( 0 < step_x ) {
      speed_x_counter--;
      if( speed_x_counter == 0 ) {
        digitalWrite(MOTOR_X_CLK, HIGH);
        step_x--;
        if( step_x == 0 ) {
          stat_x = STATUS_STOP;
        }
        speed_x_counter = speed_x;
      }
    }
  }
  if( stat_y == STATUS_RUN ) {
    if( 0 < step_y ) {
      speed_y_counter--;
      if( speed_y_counter == 0 ) {
        digitalWrite(MOTOR_Y_CLK, HIGH);
        step_y--;
        if( step_y == 0 ) {
          stat_y = STATUS_STOP;
        }
        speed_y_counter = speed_y;
      }
    }
  }
  if( stat_z == STATUS_RUN ) {
    if( 0 < step_z ) {
      speed_z_counter--;
      if( speed_z_counter == 0 ) {
        digitalWrite(MOTOR_Z_CLK, HIGH);
        step_z--;
        if( step_z == 0 ) {
          stat_z = STATUS_STOP;
        }
        speed_z_counter = speed_z;
      }
    }
  }
  delayMicroseconds(30);
  digitalWrite( MOTOR_X_CLK, LOW );
  digitalWrite( MOTOR_Y_CLK, LOW );
  digitalWrite( MOTOR_Z_CLK, LOW );
}  

//-----------------------------------------------------------------
// arduino setup
//-----------------------------------------------------------------
void setup()
{                
  pinMode(MOTOR_X_CLK,    OUTPUT);     
  pinMode(MOTOR_Y_CLK,    OUTPUT); 
  pinMode(MOTOR_Z_CLK,    OUTPUT); 
  pinMode(MOTOR_X_RESET,  OUTPUT);     
  pinMode(MOTOR_Y_RESET,  OUTPUT);     
  pinMode(MOTOR_Z_RESET,  OUTPUT);     
  pinMode(MOTOR_X_ENABLE, OUTPUT);     
  pinMode(MOTOR_Y_ENABLE, OUTPUT);     
  pinMode(MOTOR_Z_ENABLE, OUTPUT);     
  pinMode(MOTOR_X_CWCCW,  OUTPUT);
  pinMode(MOTOR_Y_CWCCW,  OUTPUT);
  pinMode(MOTOR_Z_CWCCW,  OUTPUT);
  pinMode(MOTOR_X_TQ1,    OUTPUT);
  pinMode(MOTOR_Y_TQ1,    OUTPUT);
  pinMode(MOTOR_Z_TQ1,    OUTPUT);
  pinMode(MOTOR_X_TQ2,    OUTPUT);
  pinMode(MOTOR_Y_TQ2,    OUTPUT);
  pinMode(MOTOR_Z_TQ2,    OUTPUT);
  
  digitalWrite(MOTOR_X_CLK,    LOW);
  digitalWrite(MOTOR_Y_CLK,    LOW);
  digitalWrite(MOTOR_Z_CLK,    LOW);
  digitalWrite(MOTOR_X_RESET,  LOW);
  digitalWrite(MOTOR_Y_RESET,  LOW);
  digitalWrite(MOTOR_Z_RESET,  LOW); 
  digitalWrite(MOTOR_X_ENABLE, LOW); 
  digitalWrite(MOTOR_Y_ENABLE, LOW); 
  digitalWrite(MOTOR_Z_ENABLE, LOW);
  digitalWrite(MOTOR_X_CWCCW,  LOW);
  digitalWrite(MOTOR_Y_CWCCW,  LOW);
  digitalWrite(MOTOR_Z_CWCCW,  LOW);
  digitalWrite(MOTOR_X_TQ1,    HIGH);
  digitalWrite(MOTOR_Y_TQ1,    LOW);
  digitalWrite(MOTOR_Z_TQ1,    LOW);
  digitalWrite(MOTOR_X_TQ2,    HIGH); 
  digitalWrite(MOTOR_Y_TQ2,    LOW);
  digitalWrite(MOTOR_Z_TQ2,    LOW); 

  step_x = 0;
  step_y = 0;
  step_z = 0;
  speed_x = 0;
  speed_y = 0;
  speed_z = 0;
  speed_x_counter = 0;
  speed_y_counter = 0;
  speed_z_counter = 0;
  rotation_direction_x = 'M';
  rotation_direction_y = 'M';
  rotation_direction_z = 'M';
  stat_x = STATUS_STOP;
  stat_y = STATUS_STOP;
  stat_z = STATUS_STOP;

  Serial.begin(115200);
  interpret_progress = false;

  MsTimer2::set(1, move_step);
  MsTimer2::start();
}

//-----------------------------------------------------------------
// arduino loop
//-----------------------------------------------------------------
void loop()                     
{
  exec_command();
}

// ------------ E N D ---------------------------------------------

